import os, sys, shutil, gzip
sys.path.append( "./src" )

def copyfile( srcfile, dstfile ):
    dstdir = os.path.split(dstfile)[0]
    if not os.path.isdir( dstdir ):
        os.makedirs( dstdir )
    shutil.copy( srcfile, dstfile )

def copyfiles( srcdir, dstdir ):
    if not os.path.isdir( dstdir ):
        os.makedirs( dstdir )
    for file in os.listdir( srcdir ):
        if not os.path.isdir( srcdir + "/" + file ):
            shutil.copy( srcdir + "/" + file, dstdir + "/" + file )

def copygzip( srcfile, dstfile ):
    os.system("gzip "+srcfile )
    shutil.copy( srcfile+".gz", dstfile )
    os.system("gzip -d "+srcfile+".gz")
    
def copypacks( repo_root, dstdir ):
    for wdifile in os.listdir( repo_root ): 
        if not wdifile.endswith(".wdi"): 
            continue
        shutil.copy( repo_root + wdifile, dstdir + wdifile )

def usage( error=None ):
    if error:
        print "Error: " + error
    print "Usage: python setup.py [command] <options>"
    print " Commands:"
    print "   install - installs wine-doors"
    print " uninstall - uninstalls wine-doors"
    print " remove-all - Remove every wine-doors previews version (only use if you know what you are doing)"
    print "    --help - display this usage message (default)\n"
    print " Options:"
    print "  --prefix - prefix path default ~/.local/ (ex: /)"
    #print "  --temp - Used for non final location (if you are not sure don't use it)"
    #print "  -S - make a system install in prefix for packagers"
    print
    print " To run winedoors with no install, run 'python src/winedoors.py'"
    sys.exit(0)
    pass
    
def symlink( srclink, dstlink ):
    dstdir = os.path.split(dstlink)[0]
    if not os.path.isdir( dstdir ):
        os.makedirs( dstdir )
    if not os.path.islink( dstlink ):
        os.symlink( srclink, dstlink )

if __name__ == "__main__":
    command = None
    os.chdir(sys.path[0])
    
    # Set Default options (user install):
    wine_doors = "wine-doors" # Can be used to create branches
    prefix = os.path.expanduser( "~/" )
    sysinstall = False
    temp = ""
    
    # Set Defaults case root install
    try:
        if os.environ['USERNAME'] == "root":
            prefix = "/" 
            sysinstall = True
    except:
        # Assume root, debuild fails
        prefix = "/" 
        sysinstall = True
        
    # Read command line arguments
    for argument in sys.argv:
        if argument == "install" or argument == "uninstall":
            command = argument
            continue

        if argument == "remove-all":
            command = argument
            continue

        if argument.startswith( "--temp" ):
            temp = argument.split( "=" )[1]
            continue

        if argument.startswith( "--prefix" ):
            prefix = argument.split( "=" )[1]
            continue            
                    
        if argument.startswith( "--help" ):
            usage()

        if argument.startswith( "-S" ):
            sysinstall = True

    if sysinstall:
        final = prefix
        prefix = temp + prefix
        prefix_bin = prefix + "/usr/bin/"
        prefix_data = prefix + "/usr/share/"
        prefix_conf = prefix + "/etc/" + wine_doors + "/"
        winedoors_data = prefix_data + wine_doors + "/"

        final_data = final + "usr/share/wine-doors/"
        final_conf = prefix_conf
    else:
        prefix_bin = prefix + "bin/"
        prefix_data = prefix + ".local/share/"
        prefix_conf = prefix + ".wine/wine-doors/"
        winedoors_data = prefix_data + wine_doors + "/"

        final_data = winedoors_data
        final_conf = prefix_conf
    
    if command in ("install", "uninstall"):
        from preferences import preferences
        
    if command == "install":
        if sysinstall:
            print "Performing system install with parameters: " + final + "  " + prefix
            
        if not os.path.isdir( prefix_conf ):
            os.makedirs( prefix_conf )
        conf_file = prefix_conf + "/preferences.xml"

        #Create directories
        if not os.path.isdir( prefix_data ):
            os.makedirs( prefix_data )
        if not os.path.isdir( winedoors_data ):
            os.makedirs( winedoors_data )
        if not os.path.isdir( prefix_bin ):
            os.makedirs( prefix_bin )
        
        copyfile( "./dist/wine-doors.desktop",  prefix_data + "/applications/wine-doors.desktop" )
        if os.path.isfile( prefix_data + "/applications/wine-doors.desktop" ):
            desktop = open( prefix_data + "/applications/wine-doors.desktop" , 'a' )
            if sysinstall:
                desktop.write( 'Exec=wine-doors\n' )
                desktop.write( 'Icon=wine-doors\n' )
            else:
                desktop.write( 'Exec=' + prefix_bin + "wine-doors\n" )
                desktop.write( 'Icon=' + winedoors_data + "pixmaps/wine-doors.png\n" )
            desktop.close()
      
         # Preparing and install the repositories
        #rint "Compressing and installing pack files"
        if not os.path.isdir( winedoors_data + "base.repo" ):
            os.makedirs( winedoors_data + "base.repo/" )
        #if not os.path.isdir( winedoors_data + "global.repo" ):
        #    os.makedirs( winedoors_data + "global.repo" )
        #if not os.path.isdir( winedoors_data + "games.repo" ):
        #    os.makedirs( winedoors_data + "games.repo" )

        # Execute scripts to create the packages
        #cwd = os.getcwd()
        #os.chdir( cwd + "/repos/base.repo/" )
        #print "  ** Running " + cwd + "/src/create-packs.sh"
        #print "  ** From folder " + cwd + "/repos/base.repo/"
        #os.system( cwd + "/src/create-packs.sh > /dev/null" )

        #os.chdir( cwd + "/repos/global.repo/" )
        #print "  ** Running " + cwd + "/src/create-packs.sh"
        #print "  ** From folder " + cwd + "/repos/global.repo/"
        #os.system( cwd + "/src/create-packs.sh > /dev/null" )

        #os.chdir( cwd + "/repos/games.repo/" )
        #print "  ** Running " + cwd + "/src/create-packs.sh"
        #print "  ** From folder " + cwd + "/repos/global.repo/"
        #os.system( cwd + "/src/create-packs.sh > /dev/null" )
        #os.chdir( cwd )
        
        # Install the Repositories
        print "Installing system base libraries"
        copygzip( "./repos/base.repo/packlist.xml", winedoors_data + "base.repo/packlist.xml.gz" )
        copypacks( "./repos/base.repo/", winedoors_data + "base.repo/" )

        #print "  ** Global Repo"
        #copygzip( "./repos/global.repo/packlist.xml", winedoors_data + "global.repo/packlist.xml.gz" )
        #copypacks( "./repos/global.repo/", winedoors_data + "global.repo/" )

        #print "  ** Games Repo"
        #copygzip( "./repos/games.repo/packlist.xml", winedoors_data + "games.repo/packlist.xml.gz" )
        #copypacks( "./repos/games.repo/", winedoors_data + "games.repo/" )
        
        # Install sources
        copyfiles( "./src", winedoors_data + "src" )
        os.system( "chmod +x " + winedoors_data + "src/winedoors.py" )
        copyfiles( "./pixmaps", winedoors_data + "pixmaps/" )
        copyfiles( "./registry", winedoors_data + "registry/" )

        # install symlinks
        if sysinstall:
            print "Symlinking icons"
            if not os.path.isdir( prefix_data + "/pixmaps/" ):
                os.makedirs( prefix_data + "/pixmaps/" )
            symlink( final_data + "pixmaps/wine-doors.png", prefix_data + "/pixmaps/wine-doors.png" )
            symlink( final_data + "pixmaps/wine-doors.svg", prefix_data + "/pixmaps/wine-doors.svg" )
        
        print "Symlinking executable"
        symlink( final_data + "src/winedoors.py", prefix_bin + "/wine-doors" )
        
        # Fix perms
        if sysinstall:
            print "Setting permissions"
            os.system( "chmod -R og+r "+winedoors_data )
        
        # Write preferences.xml
        print "Creating initial preferences"
        preferences.Default()
        preferences.preferences['prefix'] = final_data
        preferences.preferences['wineroot'] = ""
        preferences.AddRepo( "System Base", final_data +  "base.repo/" )
        preferences.AddRepo( "Applications", "http://www.wine-doors.org/repositories/applications.repo/" )
        preferences.AddRepo( "Libraries and Fonts", "http://www.wine-doors.org/repositories/libraries.repo/" )
        preferences.AddRepo( "Games", "http://www.wine-doors.org/repositories/games.repo/" )
        preferences.Save( conf_file )
        
        print "Checking dependencies"
        print "  wine . . . ",
        if os.popen( "which wine" ).read().rstrip() != "":
            print "Found"
        else:
            print "Not found !"
        print "  cabextract . . . ",
        if os.popen( "which cabextract" ).read().rstrip() != "":
            print "Found"
        else:
            print "Not found !"
        print "  tar . . . ",
        if os.popen( "which tar" ).read().rstrip() != "":
            print "Found"
        else:
            print "Not found !"
        print "  orange . . . ",
        if os.popen( "which orange" ).read().rstrip() != "":
            print "Found"
        else:
            print "Not found !"
        print "  ps . . . ",
        if os.popen( "which ps" ).read().rstrip() != "":
            print "Found"
        else:
            print "Not found !"
        print "  pygtk . . . ",
        try:
            import gtk, gobject
            hasui = True
            print "Found"
        except:
            print "Not found"
            hasui = False

        print "  pycairo . . . ",
        try:
            import cairo
            hasui = True
            print "Found"
        except:
            print "Not found"
            hasui = False

        print "  rsvg . . . ",
        try:
            import rsvg
            hasui = True
            print "Found"
        except:
            print "Not found"
            hasui = False

        if not hasui:
            print "One of the required modules for the wine-doors user interface is missing"
            print "the wine-doors command line interface is available, for more information" 
            print "try running: " + prefix_bin + "/wine-doors --help"

    elif command == "uninstall":
        if sysinstall:
            print "Removing system wide preferences"
            if os.path.exists( prefix_conf ):
                shutil.rmtree( prefix_conf )
        
        print "Removing symlinks"
        
        if sysinstall:
            if os.path.exists( prefix_data + "pixmaps/wine-doors.png" ):
                os.remove( prefix_data + "pixmaps/wine-doors.png" )
            if os.path.exists( prefix_data + "pixmaps/wine-doors.svg" ):
                os.remove( prefix_data + "pixmaps/wine-doors.svg" )

        if os.path.exists( prefix_bin + "/wine-doors" ):
            os.remove( prefix_bin + "/wine-doors" )
            
        print "Removing menu item"
        if os.path.exists( prefix_data + "applications/wine-doors.desktop" ):
            os.remove ( prefix_data + "applications/wine-doors.desktop" )
        print "Removing shared data"
        if os.path.isdir( winedoors_data ):
            shutil.rmtree( winedoors_data, True)
            
    elif command == "remove-all":
        if os.path.exists( os.path.expanduser( "~/.local/share/pixmaps/wine-doors.png" )):
            os.remove( os.path.expanduser( "~/.local/share/pixmaps/wine-doors.png" ))
        if os.path.exists( os.path.expanduser( "~/.local/share/pixmaps/wine-doors.svg" )):
            os.remove( os.path.expanduser( "~/.local/share/pixmaps/wine-doors.svg" ))

        if os.path.exists( os.path.expanduser( "~/.wine-doors" )):
            shutil.rmtree( os.path.expanduser( "~/.wine-doors" ), True )

        if os.path.exists( os.path.expanduser( "~/.wine" )):
            shutil.rmtree( os.path.expanduser( "~/.wine" ), True )
            
        print "Removing menu item"
        if os.path.exists( os.path.expanduser( "~/.local/share/applications/wine-doors.desktop" )):
            os.remove ( os.path.expanduser( "~/.local/share/applications/wine-doors.desktop" ))
        if os.path.isdir( os.path.expanduser( "~/.local/share/applications/wine" )):
            shutil.rmtree( os.path.expanduser( "~/.local/share/applications/wine" ))
        print "Removing shared data"
        if os.path.isdir( os.path.expanduser( "~/.local/share/wine-doors" )):
            shutil.rmtree( os.path.expanduser( "~/.local/share/wine-doors" ), True)

        # Removing root install
        if os.path.exists( "/usr/share/pixmaps/wine-doors.png" ):
            os.remove( "/usr/share/pixmaps/wine-doors.png" )
        if os.path.exists( "/usr/share/pixmaps/wine-doors.svg" ):
            os.remove( "/usr/share/pixmaps/wine-doors.svg" )

        if os.path.isdir( "/etc/wine-doors" ):
            shutil.rmtree( "/etc/wine-doors", True )

        print "Removing menu item"
        if os.path.exists( "/usr/share/applications/wine-doors.desktop" ):
            os.remove ( "/usr/share/applications/wine-doors.desktop" )
        print "Removing shared data"
        if os.path.isdir( "/usr/share/wine-doors" ):
            shutil.rmtree( "/usr/share/wine-doors", True)

    else:
        usage("No command specified")
